# frozen_string_literal: true

Dir[File.dirname(__FILE__)].each { |file| $LOAD_PATH.unshift(file) if File.directory? file }

require 'logging'
require 'eventmachine'
require 'sqlite3'
require 'rufus/scheduler'
require 'ostruct'

module RuneRb
  
  module Engine
    autoload :EventManager,     'core/engine'
    autoload :Event,            'core/engine'
    autoload :QueuePolicy,      'core/engine'
    autoload :WalkablePolicy,   'core/engine'
    autoload :Action,           'core/engine' # TODO move to Actions
    autoload :ActionQueue,      'core/engine' # TODO move to Actions
  end
  
  module Misc
    autoload :AutoHash,            'core/util'
    autoload :HashWrapper,         'core/util'
    autoload :Flags,               'core/util'
    autoload :TextUtils,           'core/util'
    autoload :NameUtils,           'core/util'
    autoload :ThreadPool,          'core/util'
    autoload :Cache,               'core/cache'
  end
  
  module Actions
    autoload :HarvestingAction,    'core/actions'
  end
  
  module Model
    autoload :HitType,             'model/combat'
    autoload :Hit,                 'model/combat'
    autoload :Damage,              'model/combat'
    autoload :Animation,           'model/effects'
    autoload :Graphic,             'model/effects'
    autoload :ChatMessage,         'model/effects'
    autoload :Entity,              'model/entity'
    autoload :Location,            'model/location'
    autoload :Player,              'model/player'
    autoload :RegionManager,       'model/region'
    autoload :Region,              'model/region'
  end
  
  module Item
    autoload :Item,                       'model/item'
    autoload :ItemDefinition,             'model/item'
    autoload :Container,                  'model/item'
    autoload :ContainerListener,          'model/item'
    autoload :InterfaceContainerListener, 'model/item'
    autoload :WeightListener,             'model/item'
    autoload :BonusListener,              'model/item'
  end
  
  module NPC
    autoload :NPC,                 'model/npc'
    autoload :NPCDefinition,       'model/npc'
  end
  
  module Player
    autoload :Appearance,          'model/player/appearance'
    autoload :InterfaceState,      'model/player/interfacestate'
    autoload :RequestManager,      'model/player/requestmanager'
    autoload :Skills,              'model/player/skills'
  end
  
  module Net
    autoload :Server,              'net/server'
    autoload :ActionSender,        'net/actionsender'
    autoload :ISAAC,               'net/isaac'
    autoload :Session,             'net/session'
    autoload :Connection,          'net/connection'
    autoload :Packet,              'net/packet'
    autoload :PacketBuilder,       'net/packetbuilder'
    autoload :JaggrabConnection,   'net/jaggrab'
  end
  
  module GroundItems
    autoload :GroundItem,          'services/ground_items'
    autoload :GroundItemEvent,     'services/ground_items'
    autoload :PickupItemAction,    'services/ground_items'
  end
  
  module Shops
    autoload :ShopManager,         'services/shops'
    autoload :Shop,                'services/shops'
  end
  
  module Objects
    autoload :ObjectManager,       'services/objects'
  end
  
  module Doors
    autoload :DoorManager,         'services/doors'
    autoload :Door,                'services/doors'
    autoload :DoubleDoor,          'services/doors'
    autoload :DoorEvent,           'services/doors'
  end
  
  module Tasks
    autoload :NPCTickTask,         'tasks/npc_update'
    autoload :NPCResetTask,        'tasks/npc_update'
    autoload :NPCUpdateTask,       'tasks/npc_update'
    autoload :PlayerTickTask,      'tasks/player_update'
    autoload :PlayerResetTask,     'tasks/player_update'
    autoload :PlayerUpdateTask,    'tasks/player_update'
    autoload :SystemUpdateEvent,   'tasks/sysupdate_event'
  end
  
  module World
    autoload :Profile,             'world/profile'
    autoload :Pathfinder,          'world/walking'
    autoload :Point,               'world/walking'
    autoload :World,               'world/world'
    autoload :LoginResult,         'world/world'
    autoload :Loader,              'world/world'
    autoload :YAMLFileLoader,      'world/world'
    autoload :NPCSpawns,           'world/npc_spawns'
    autoload :ItemSpawns,          'world/item_spawns'
  end
end

require 'core/plugin_hooks'
require 'net/packetloader'

